#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            configuration.py
#
# Licence:         Donationware, distributed as Freeware
#
#                  If you find this software useful to you and you would
#                  like to help the developer to maintain it also in the future, 
#                  please donate a small amount of your liking to the 
#                  PayPal account: olammi@iki.fi.
#
#                  This donation is not required to use the software and the 
#                  software is licenced as freeware.
#
#                  To individual and non-commercial use only! Script
#                  and source code may be used freely for non-commercial
#                  purposes. Any commercial usage requires permission or
#                  licence from the author. 
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0a
#
# Date:            06.04.2009
#
# Description:     Configuration file class for taloLogger.py
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
#
# Version history: ** 24.02.2009 v0.9a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 24.02.2009 v0.9a (Olli Lammi) **
#                  Added methods for checking the configuration. 
#
###########################################################################

# Imports

import sys, os, string


###########################################################################

# Classes

class Configurable:
    # returns a tuple with two lists both containing the
    # allowed normal and listed configuration keys that this
    # configurable allows
    @staticmethod
    def getAllowedConfigurationKeys(): pass

    # reads configuration conf and configurest itself
    # returns a tuple (stat, msg) where msg is possible
    # error message and stat is (1 = ok, 0 = error)
    def handleConfiguration(self, conf): pass

    # initializes the configurable with the given config ie. runs appropriate tests
    # to check if the configuration is semantically valid and all configured resources
    # work accordingly.
    # returns a tuple (stat, msg) where msg is possible error message
    # and stat is (1 = ok, 0 = error)
    def initConfiguration(self):
        return (1, '')


class Configuration:
    def __init__(self):
        self.data = {}
        self.allowedkeys = []
        self.allowedlistkeys = []
        
    def addAllowedKeys(self, keys):
        for k in keys:
            self.allowedkeys.append(k)

    def addAllowedListKeys(self, keys):
        for k in keys:
            self.allowedlistkeys.append(k)

    def addConfigurable(self, cfable):
        (norm, lst) = cfable.getAllowedConfigurationKeys()
        self.addAllowedKeys(norm)
        self.addAllowedListKeys(lst)

    # load configuration file: tuple stat, msg: 0 = open/read error, -1 = invalid file, 1 = ok
    def loadFile(self, fname):
        try:
            inf = open(fname, 'r')
            lines = inf.readlines()
            inf.close()
        except:
            return (0, '')

        for line in lines:
            line = string.strip(string.split(line, '\n')[0])
            if len(line) <= 0 or line[0] == '#':
                continue

            if line[0] == '@':
                line = line[1:]
                lineparts = string.split(line, '=', 1)
                if len(lineparts) > 1:
                    key = string.strip(lineparts[0])
                    if not key in self.allowedlistkeys:
                        return (-1, "Invalid configuration key: @" + key)
                    value = string.strip(lineparts[1])
                    if not self.data.has_key(key):
                        self.data[key] = [] 
                    self.data[key].append(value)
                else:
                    return (-1, "Invalid configuration line: @" + line)
            else:
                lineparts = string.split(line, '=', 1)
                if len(lineparts) > 1:
                    key = string.strip(lineparts[0])
                    if not key in self.allowedkeys:
                        return (-1, "Invalid configuration key: " + key)
                    value = string.strip(lineparts[1])
                    self.data[key] = value
                else:
                    return (-1, "Invalid configuration line: " + line)

        return (1, '')

    def getData(self):
        return self.data

    def hasKey(self, key):
        return self.data.has_key(key)

    def getValue(self, key, default):
        if self.hasKey(key):
            return self.data[key]
        else:
            return default

    def setValue(self, key, value):
        self.data[key] = value

    def isTrue(self, key):
        if not self.hasKey(key):
            return 0
        if string.lower(self.data[key]) == 'true':
            return 1
        return 0
