#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            storeDbSQLite.py
#
# Licence:         Donationware, distributed as Freeware
#
#                  If you find this software useful to you and you would
#                  like to help the developer to maintain it also in the future, 
#                  please donate a small amount of your liking to the 
#                  PayPal account: olammi@iki.fi.
#
#                  This donation is not required to use the software and the 
#                  software is licenced as freeware.
#
#                  To individual and non-commercial use only! Script
#                  and source code may be used freely for non-commercial
#                  purposes. Any commercial usage requires permission or
#                  licence from the author. 
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0a
#
# Date:            22.10.2010
#
# Description:     Data Store class to store logged data values into a
#                  SQLite file based database.
#                  
# Requirements:    Python interpreter 2.5 or newer (www.python.org)
#                  (tested with 2.7)
# 
#                  MySQL-python library (MySQLdb) 
#                  (http://sourceforge.net/projects/mysql-python)
#
# Version history: ** 22.10.2010 v1.0a (Olli Lammi) **
#                  SQLite support added. 
#
###########################################################################

# Imports

import sys, os 
import string

import store, configuration
import log

# sqlite3 module, not loaded until init of DBStoreSQLite class instance
SQLiteDb = None

###########################################################################

# Classes

class DBStoreSQLite(store.Store, configuration.Configurable, log.Logging):
    def __init__(self, conf):
        log.Logging.__init__(self, 'DBStoreSQLite')

        global SQLiteDb
        SQLiteDb = __import__('sqlite3') 

        self.handleConfiguration(conf)

    def handleConfiguration(self, conf):
        self.DB_NAME = conf.getValue('SQLITEDB_NAME', '')
        self.DB_TABLE = conf.getValue('SQLITEDB_TABLE', '')
        self.DB_TIMECOL = conf.getValue('SQLITEDB_TIMECOL', '')        
        return (1, '')

    def initConfiguration(self):
        status = 0
        sqlstmt = ""
        try:
            db = SQLiteDb.connect(self.DB_NAME)
            cur = db.cursor()
            sqlstmt = "SELECT COUNT(" + self.DB_TIMECOL + ") FROM " + self.DB_TABLE
            cur.execute(sqlstmt)
            cur.close()
            db.close()
            status = 1
        except:
            self.Log("ERROR: Error in database operation, SQL: " + sqlstmt)
            status = 0
        if status:
            return (1, '')
        else:
            return (0, 'DBStoreSQLite: Errors with database access.')

 
    def insertData(self, values):
        sqlstmt = ""
        try:
            db = SQLiteDb.connect(self.DB_NAME)
            cur = db.cursor()

            columnstr = ""
            valuestr = ""
            for item in values:
                if len(columnstr) > 0:
                    columnstr = columnstr + ", "
                if len(valuestr) > 0:
                    valuestr = valuestr + ", "
                columnstr = columnstr + item[0]
                valuestr = valuestr + checkDBValue(item[1])
                
            sqlstmt = "INSERT INTO " + self.DB_TABLE + " "
            sqlstmt = sqlstmt + "(" + self.DB_TIMECOL + ", "
            sqlstmt = sqlstmt + columnstr + ") VALUES (DATETIME('NOW'), "
            sqlstmt = sqlstmt + valuestr + ")"

            self.Debug("Inserting data to database using SQL: " + sqlstmt)
            cur.execute(sqlstmt)
            cur.close()
            db.commit()
            db.close()
        except:
            self.Log("ERROR: Error in database operation, SQL: " + sqlstmt)

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['SQLITEDB_NAME', 'SQLITEDB_TABLE', 'SQLITEDB_TIMECOL'], [])


###########################################################################

# Functions

def checkDBValue(val):
    if (val and len(val) > 0):
        return val
    return "NULL"

