#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            storeFile.py
#
# Licence:         Donationware, distributed as Freeware
#
#                  If you find this software useful to you and you would
#                  like to help the developer to maintain it also in the future, 
#                  please donate a small amount of your liking to the 
#                  PayPal account: olammi@iki.fi.
#
#                  This donation is not required to use the software and the 
#                  software is licenced as freeware.
#
#                  To individual and non-commercial use only! Script
#                  and source code may be used freely for non-commercial
#                  purposes. Any commercial usage requires permission or
#                  licence from the author. 
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0a
#
# Date:            06.04.2009
#
# Description:     Data Store class to store logged data values into a
#                  file.
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 24.02.2009 v0.9a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 25.02.2009 v0.9c (Olli Lammi) **
#                  Logging and import changes 
#
#                  ** 26.02.2009 v0.9d (Olli Lammi) **
#                  Support for patterns in configuration. 
#
#                  ** 06.04.2009 v1.0a (Olli Lammi) **
#                  Changed to support configurable init. 
#
###########################################################################

# Imports

import sys, os 
import string, time

import store, log, configuration


###########################################################################

# Classes

class FileStore(store.Store, configuration.Configurable, log.Logging):
    def __init__(self, conf):
        log.Logging.__init__(self, 'FileStore')

        self.handleConfiguration(conf)        

    def handleConfiguration(self, conf):
        self.filename = conf.getValue('FILESTORE_NAME', '')
        self.timepat = conf.getValue('FILESTORE_TIMESTAMP', '')
        self.header = conf.getValue('FILESTORE_HEADER', '')
        self.dataval = conf.getValue('FILESTORE_DATAVALUE', '')
        self.footer = conf.getValue('FILESTORE_FOOTER', '')
        
        if len(self.filename) <= 0:
            return (0, "Missing file store filename.")
        return (1, '')

    def initConfiguration(self):
        status = 0
        try: 
            outf = open(self.filename, 'a')
            outf.close()
            status = 1
        except:
            status = 0
        if status:
            return (1, '')
        else:
            return (0, 'FileStore: Error opening file store: ' + self.filename)

    def insertData(self, values):
        tstr = ''
        headerstr = ''
        footerstr = ''
        valstr = ''
        
        try:
            tstr = time.strftime(self.timepat)
        except:
            self.Log("ERROR: Error creating timestamp using pattern: " + self.timepat)
            
        headerstr = ReplacePatterns(self.header, tstr, '', '')
        footerstr = ReplacePatterns(self.footer, tstr, '', '')
        for item in values:
            valstr = valstr + ReplacePatterns(self.dataval, tstr, item[0], item[1])
        
        self.Debug("Inserting data to file: " + self.filename)
        try: 
            outf = open(self.filename, 'a')
            outf.write(headerstr + valstr + footerstr)
            outf.close()
        except:
            self.Log("ERROR: Error writing data store file: " + self.filename)

    @staticmethod
    def getAllowedConfigurationKeys():
        return (['FILESTORE_NAME', 'FILESTORE_TIMESTAMP', 'FILESTORE_HEADER', \
                 'FILESTORE_DATAVALUE', 'FILESTORE_FOOTER'], [])


###########################################################################

# Functions

def ReplacePatterns(pattern, timestr, key, data):
    temps = pattern
    temps = ReplaceStrCI(temps, '%S%', ' ')
    temps = ReplaceStrCI(temps, '%N%', '\n')
    temps = ReplaceStrCI(temps, '%T%', '\t')
    temps = ReplaceStrCI(temps, '%D%', '\x0D')
    temps = ReplaceStrCI(temps, '%A%', '\x0A')
    temps = ReplaceStrCI(temps, '%TIME%', timestr)
    temps = ReplaceStrCI(temps, '%KEY%', key)
    temps = ReplaceStrCI(temps, '%VALUE%', data)
    temps = ReplaceStrCI(temps, '%VALUE_C%', string.replace(data, '.', ','))
    temps = ReplaceStrCI(temps, '%VALUE_P%', string.replace(data, ',', '.'))
    return temps

def ReplaceStrCI(str, pat, wstr):
    ind = 0 
    tstr = str
    while ind >= 0:
        ind = string.find(string.upper(tstr), string.upper(pat))
        if ind >= 0:
           tstr = tstr[:ind] + wstr + tstr[ind+len(pat):] 
    return tstr
