#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            threads.py
#
# Licence:         Donationware, distributed as Freeware
#
#                  If you find this software useful to you and you would
#                  like to help the developer to maintain it also in the future, 
#                  please donate a small amount of your liking to the 
#                  PayPal account: olammi@iki.fi.
#
#                  This donation is not required to use the software and the 
#                  software is licenced as freeware.
#
#                  To individual and non-commercial use only! Script
#                  and source code may be used freely for non-commercial
#                  purposes. Any commercial usage requires permission or
#                  licence from the author. 
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0a
#
# Date:            24.02.2009
#
# Description:     Library to implement threading and locking classes in
#                  python.
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 02.12.2008 v1.0a (Olli Lammi) **
#                  First version. 
#
###########################################################################

# Imports

import sys
import thread, signal, time


###########################################################################

# Classes

class Lock:
    def __init__(self):
        self.mylock = thread.allocate_lock()

    def lock_wait(self):
        return self.mylock.acquire()

    def lock_immediate(self):
        return self.mylock.acquire(0)

    def free(self):
        return self.mylock.release()

class Thread:
    def __init__(self):
        self.lock = Lock()
        self.tid = None

    def start(self):
        if self.lock.lock_immediate():
            try:
                self.tid = thread.start_new_thread(self.main, (self,))
            except:
                self.lock.free()
                return 0
            return 1
        else:
            return 0

    def main(self, starter):
        try:
            starter.run()
        except:
            print "Error running thread (thread id " + `self.tid` + "):", sys.exc_info()[0], sys.exc_info()[1]
            sys.exc_clear()
        starter.lock.free()        

    def run(self):
        print "Unimplemented thread run method."

    def wait(self):
        self.lock.lock_wait()
        self.lock.free()

    def wait_sleeping(self):
        while 1:
            if self.lock.lock_immediate():
               self.lock.free()
               return 
            else:
               time.sleep(1)
